"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
function sync(fsAdapter, root, options) {
    const names = fsAdapter.readdir(root);
    const entries = [];
    for (const name of names) {
        if (options.preFilter && !options.preFilter(name)) {
            continue;
        }
        const fullPath = path.join(root.toString(), name);
        const stats = fsAdapter.stat(fullPath, {
            followSymlinks: options.followSymlinks,
            throwErrorOnBrokenSymlinks: options.throwErrorOnBrokenSymlinks
        });
        const entry = makeDirEntry(name, fullPath, stats, options);
        if (options.filter && !options.filter(entry)) {
            continue;
        }
        entries.push(entry);
    }
    if (options.sort) {
        return entries.sort(options.sort);
    }
    return entries;
}
exports.sync = sync;
function async(fsAdapter, root, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let names = yield fsAdapter.readdir(root);
        if (options.preFilter) {
            names = names.filter(options.preFilter);
        }
        const fullPaths = [];
        const promises = [];
        for (const name of names) {
            const fullPath = path.join(root.toString(), name);
            const promise = fsAdapter.stat(fullPath, {
                followSymlinks: options.followSymlinks,
                throwErrorOnBrokenSymlinks: options.throwErrorOnBrokenSymlinks
            });
            fullPaths.push(fullPath);
            promises.push(promise);
        }
        const stats = yield Promise.all(promises);
        const entries = [];
        for (let index = 0; index < names.length; index++) {
            const entry = makeDirEntry(names[index], fullPaths[index], stats[index], options);
            if (options.filter && !options.filter(entry)) {
                continue;
            }
            entries.push(entry);
        }
        if (options.sort) {
            return entries.sort(options.sort);
        }
        return entries;
    });
}
exports.async = async;
function makeDirEntry(name, full, stats, options) {
    const entry = {
        name,
        path: full,
        ino: stats.ino,
        isDirectory: stats.isDirectory(),
        isFile: stats.isFile(),
        isSymlink: stats.isSymbolicLink()
    };
    if (options.stats) {
        entry.stats = stats;
    }
    return entry;
}
exports.makeDirEntry = makeDirEntry;
